library logger;

uses
  windows,
  SyncObjs,
  SysUtils,
  Graphics,
  Messages,
  Classes,
  LoggerForm in 'LoggerForm.pas' {ALoggerForm},
  LoggerSettingsForm in 'LoggerSettingsForm.pas' {SettingsForm},
  LoggerActDoUnit in 'LoggerActDoUnit.pas' {LoggerActDoForm},
  LoggerActSetUnit in 'LoggerActSetUnit.pas' {LoggerActSetForm};

type
  TLogData = array[1..12] of string;

  TExtData = array[0..255] of longint;

  {$R *.RES}

Const //    
  st_NotRun   = 0; //   
  st_Run      = 1; // 
  st_Complete = 2; //  
  st_Failed   = 3; //   
  st_Aborted  = 4; // 
  st_Retried  = 5; //   
  st_Sleep    = 6; // 
  st_Wait     = 7; // 
  st_Update   = 8; //
  st_Report   = 9; //
  st_Notify   = 10;

var
  fLogFile: TextFile;
  OnlyOneThread: TCriticalSection;
  fCount: Longint;
  fAssigned : boolean;
  EnableLog : integer;
  hMainWND : HWND;
  ABitmap : TBitmap;

//    
//  
function GetStrValue(Section : TStrings; ValID : string) : string;
var
  i : integer;
begin
  Result := '';
  for i := 0 to Section.Count - 1 do begin
    if Pos(ValID+'=',Section[i]) > 0 then begin
      Result := StringReplace(Section[i], ValID+'=', '',[rfReplaceAll]);
      break;
    end;
  end
end;




  ///////////////////////////////////////////
  //   
  // (   . )
  ///////////////////////////////////////////
function _ActivateLogger : integer;
begin
    if fAssigned then exit;
    try //  
      AssignFile(fLogFile, fLogFileName);
      //fActive := true;
      if FileExists(fLogFileName) then begin
        Append(fLogFile);
        fAssigned := true;
      end else begin
        ReWrite(fLogFile);
        fAssigned := true;
        fCount    := 0;
      end;
    except
      try
        ReWrite(fLogFile);
        fAssigned := true;
        //fActive   := true;
        fCount    := 0;
      except
        fActive   := false;
      end;
    end;
end;

function _DeActivateLogger : integer;
begin
end;

function _InitLogger : integer;
begin
  //  
  fActive   := false;
  fAssigned := false;
  OnlyOneThread := TCriticalSection.Create;
  try
    ALoggerForm := TALoggerForm.Create(nil);
    ALoggerActSetForm := TLoggerActSetForm.Create(nil);
    ALoggerForm.StringGrid1.RowCount := 1;
  except end;
  try
    ASettingsForm := TSettingsForm.Create(nil);
  except end;

end;

  ///////////////////////////////////////////
  //   
  ///////////////////////////////////////////

function _CloseLogger : integer;
begin
  fActive := false;
  if fAssigned then
  try
    CloseFile(fLogFile);
  except
  end;
  fAssigned := false;
  //ALoggerForm.StringGrid1.Clear;
  ALoggerForm.Free;   ALoggerForm   := Nil;
  ASettingsForm.Free; ASettingsForm := Nil;
  //ALoggerActDoForm.Close;
  //ALoggerActDoForm.Free;
  //ALoggerActDoForm := Nil;
  ALoggerActSetForm.Free;
  ALoggerActSetForm := Nil;
  OnlyOneThread.Free; OnlyOneThread := Nil;
end;

  ///////////////////////////////////////////
  //     
  ///////////////////////////////////////////

function _LogEvent(LogData : TLogData) : integer;
var
  i : integer;
  NewName, S : string;
begin
//  exit;

  if not fActive or not fAssigned then exit;
  OnlyOneThread.Enter;
  try
      if FileSize(fLogFile) >= fMaxLogSize then begin
        CloseFile(fLogFile);
        i := 1;
        Repeat
          NewName := ChangeFileExt(fLogFileName,IntToStr(i)+ExtractFileExt(fLogFileName));
          i:= i + 1;
        Until not FileExists(NewName);
        RenameFile(fLogFileName, NewName);
        AssignFile(fLogFile, fLogFileName);
        ReWrite(fLogFile);
        fCount := 0;
      end;

      WriteLn(fLogFile,
       LogData[1]+';'
      +LogData[2]+';'
      +LogData[3]+';'
      +LogData[4]+';'
      +LogData[5]+';'
      +LogData[6]+';'
      +LogData[7]+';'
      +LogData[8]+';'
      +LogData[9]+';'
      );
{      with ALoggerForm.LogStringGrid do begin
        AddRow;
        if Row = RowCount - 2 then begin
          Row := RowCount - 1;
          ScrollInView(0, Row);
        end;
        SetString(S,PChar(LogData[1]),Length(LogData[1]));
        Cells[1,RowCount - 1] := S;
        SetString(S,PChar(LogData[2]),Length(LogData[2]));
        Cells[2,RowCount - 1] := S;
        SetString(S,PChar(LogData[3]),Length(LogData[3]));
        Cells[3,RowCount - 1] := S;
        SetString(S,PChar(LogData[8]),Length(LogData[8]));
        Cells[4,RowCount - 1] := S;
        SetString(S,PChar(LogData[10]),Length(LogData[10]));
        if S <> '' then
          AddComment(4,RowCount - 1,StringReplace(S, '<EOL>',#13#10, [rfReplaceAll]));
      end;
}
      Flush(fLogFile);
  finally
    OnlyOneThread.Leave;
  end;
end;


  ///////////////////////////////////////////
  //     
  ///////////////////////////////////////////
//  :  
//     ,
//    
function _ShowActionSettings(MyStruct : Pointer) : integer;
var
  pMyStruct : ^TMyStruct;
begin
  pMyStruct := MyStruct;
//  if
  if MyStruct <> nil then
  with ALoggerActSetForm do begin
    MyStructPonter := MyStruct;
    edCaption.Text := pMyStruct.Caption;
    edText.Text    := pMyStruct.TextFName;
    edTimeOut.Text := IntToStr(pMyStruct.Timeout);
  end;
  ALoggerActSetForm.Show;
  //Result := -1;
end;

  ///////////////////////////////////////////
  //   
  ///////////////////////////////////////////
//  :  
//     ,
//    
function _DoAction(MyStruct : Pointer) : integer;
{type
  TActDta = record
    AddOnStruct : pointer;
    CallBackFunc: pointer;
    JobStr1,
    JobStr2,
    ActResStr   : string;
    ActResult   : integer;
    ActProgress : integer;
    Break       : boolean;
  end; }
var
  ACTDATA   : ^TExtData;
  pMyStruct : ^TMyStruct;
  res : integer;
begin
  ACTDATA := MyStruct;
  pMyStruct := Pointer(ACTDATA[0]);//   AddOnStruct
             //ACTDATA[1];//   CallBackFunc
             //ACTDATA[2];//   JobStr1
             //ACTDATA[3];//   JobStr2
             //ACTDATA[4];//   ActResStr
             //ACTDATA[5];// ActResult
             //ACTDATA[6];// ActProgress
             //ACTDATA[7];//   Break : boolean
             //ACTDATA[8] //    
             //ACTDATA[9] //    

  ALoggerActDoForm  := TLoggerActDoForm.CreateParented(hMainWND);
  try
    with ALoggerActDoForm do begin
      Memo1.Clear;
      TimeOut := pMyStruct.Timeout;
      if FileExists(pMyStruct.TextFName) then
        Memo1.Lines.LoadFromFile(pMyStruct.TextFName)
      else
        Memo1.Lines.Add(pMyStruct.TextFName);
      Caption := pMyStruct.Caption;
      Timer1.Enabled := true;
    //ShowModal;
      Res := ShowModal;
      Timer1.Enabled := false;
      if Res = 1 then ACTDATA[5] := st_Complete
      else ACTDATA[5] := st_Failed;
    end;
    finally
      ALoggerActDoForm.Destroy;
    end;
  //Result := -1;
end;


  ///////////////////////////////////////////
  //    
  ///////////////////////////////////////////
function _ShowLog(BegData,EndData : TDateTime) : integer;
var
  hIcon : THandle;
begin
  ALoggerForm.hMainWND := hMainWND;
  with ALoggerForm do begin
    FromDate.Date := BegData;
    ToDate.Date := EndData;
    LogFileName1C := PChar(fLogFileName);
      //ShowWindow(hMainWND,SW_HIDE);
    Show;
    btReadLogClick(nil);
  end;
end;


  ///////////////////////////////////////////
  //     
  ///////////////////////////////////////////
function _ShowSettings : integer;
begin
  Result := 0;
  //   fLogFileName  fActive
  //    LoggerSettingsForm
  //         
  with ASettingsForm do begin
    MlgFileName.Text := fLogFileName;
    MaxLogSize.Text := IntToStr(fMaxLogSize);
    LogActive.Checked   := fActive;
    Show;
  end;
end;

  ///////////////////////////////////////////
  //     cfg -  ...
  ///////////////////////////////////////////

function _OnGetCfgSection(pCfgSectionText : PChar) : integer;
var
  Section : TStrings;
  S : String;
  P : PChar;

begin
//   
  fActive      := true;
  fMaxLogSize  := 204800;
  try
    Section := TStringList.Create; //   (  )
    try
      Section.Text := pCfgSectionText;
      fLogFileName := GetStrValue(Section,'LogFileName');
      fActive      := StrToBoolDef(GetStrValue(Section,'EnabledLog'),fActive);
      fMaxLogSize  := StrToIntDef(GetStrValue(Section,'MaxLogSize'),fMaxLogSize);
      //except end;
    finally
      Section.Free;
    end;
  except
  end;
  if ((fLogFileName) = '') and fActive then
    fLogFileName := ChangeFileExt(ParamStr(0),'.mlg');

end;

  ///////////////////////////////////////////
  //     ini -  ...
  ///////////////////////////////////////////
function _OnGetIniSection(var pParam : PChar) : integer;
var
  Section : TStrings;
  S : String;
  P : PChar;
  pMyStruct : ^TMyStruct;
  i : integer;
begin
  Section := TStringList.Create; //   (  )
//   
  New(pMyStruct);
  with pMyStruct^ do begin
    Caption   := '  ...';
    TextFName := '   ?'+#13#10+' - ""'+#13#10+' - ""';
    Timeout   := 60;
    try
      try
        Section.Text := pParam;
        Caption   := GetStrValue(Section,'MyMessageCaption');
        TextFName := GetStrValue(Section,'MyMessageText');
        Timeout   := StrToInt(GetStrValue(Section,'MyMessageTimeOut'));
      except end;
    finally
      pParam := PChar(pMyStruct);
      Section.Free;
    end;
  end;
end;

  ///////////////////////////////////////////
  //    cfg  
  //      LParamOfMsg
  ///////////////////////////////////////////

function _PrepareCfgSection(Var LParamOfMsg : LongInt) : integer;
begin
  LParamOfMsg := LongInt(PChar(
   'LogFileName='+fLogFileName+#13#10+
   'EnabledLog='+BoolToStr(fActive))+#13#10+
   'MaxLogSize='+IntToStr(fMaxLogSize)+#13#10
   );
end;


  ///////////////////////////////////////////
  //    ini  
  //      LParamOfMsg
  ///////////////////////////////////////////

function _PrepareIniSection(Var LParamOfMsg : LongInt) : integer;
var
  pMyStruct : ^TMyStruct;
begin  //   
  pMyStruct := Pointer(LParamOfMsg);
  LParamOfMsg := LongInt(PChar(
   'MyMessageCaption='+pMyStruct.Caption+#13#10+
   'MyMessageText='+pMyStruct.TextFName+#13#10+
   'MyMessageTimeOut='+IntToStr(pMyStruct.Timeout))+#13#10
   );
end;

  ///////////////////////////////////////////
  //  (   )
  //   
  ///////////////////////////////////////////
function _InitActData(Var LParamOfMsg : LongInt) : integer;
var
  pMyStruct : ^TMyStruct;
begin
  New(pMyStruct);
  LParamOfMsg := LongInt(pMyStruct);
end;
  ///////////////////////////////////////////
  //  -        
  // (   .  )    :
  // TRUE - WndProc     
  //  FALSE - WndProc     
  ///////////////////////////////////////////

function OnMainWndProc (var message: TMessage) : boolean;
var
  aPlData : TExtData;  //   
  pApData : ^TExtData; //    
  nSize : DWORD;
  lpFilename : TFileName;
begin
  Result := true;
  //MessageBox(0,'O!', 'O!', MB_OK);

  if message.Msg = WM_USER then begin
    Case message.WParam of
      1000 : begin
//     +   ( )
                  //MemoAboutPlugin.Lines.Add(pPlugin.Name);
{CompanyName      }aPlData[0] := LongInt(PChar(' "-"'));
{FileDescription  }aPlData[1] := LongInt(PChar('       .'));
{FileVersion      }aPlData[2] := LongInt(PChar('1.0.0.1'));
{InternalName     }aPlData[3] := LongInt(PChar('PnLogger'));
{LegalCopyright   }aPlData[4] := LongInt(PChar(' 2004-2005 Barinov Dmitry'));
{LegalTrademarks  }aPlData[5] := LongInt(PChar('PnLogger'));
{OriginalFilename }aPlData[6] := LongInt(PChar('logger.dll'));
{ProductName      }aPlData[7] := LongInt(PChar(' '));
{ProductVersion   }aPlData[8] := LongInt(PChar('1.0.0.1'));
{Comments         }aPlData[9] := LongInt(PChar('       : http://pnsheduler.narod.ru        .'));
{GUID             }aPlData[10]:= LongInt(PChar('F4A92098-F320-4AEB-9D41-620709D574B2'));
{CFG_SEction      }aPlData[11]:= LongInt(PChar('PnLogger'));
{HTTP             }aPlData[12]:= LongInt(PChar('http://pnsheduler.narod.ru/download.html'));
{e-mail           }aPlData[13]:= LongInt(PChar('pnsheduler@narod.ru, barinov-da@yandex.ru'));
        _InitLogger;
        message.LParam := LongInt(@aPlData);
      end;// 1000

      1001 : begin //  
        _CloseLogger;
      end;// 1001

      1002 : begin //        cfg-
        _PrepareCfgSection(message.LParam);
      end;// 1002

      1003 : begin //       cfg-
        _OnGetCfgSection(PChar(message.LParam));
      end;// 1003

      1004 : begin //  
        pApData := Pointer(message.LParam);
        hMainWND := HWND(pApData[0]);

      //         
      //    cfg
        _ActivateLogger;
      end;

      1005 : begin // - 
        _DeActivateLogger;
      end;

//   >>>>
//           ,
//        .
//   /      ini-
//  /      .
//         .
      1010 : begin //        ini-
        _PrepareIniSection(message.LParam);
      end;// 1010
      1011 : begin //       ini-
        //   (    )
        //   message.LParam
        _OnGetIniSection(PChar(message.LParam));
      end;// 1011
      1012 : begin //   ""
      end;// 1012
      1013 : begin //       
        _DoAction(Pointer(message.LParam));
      end;// 1013
      1014 : begin //    
        _ShowActionSettings(Pointer(message.LParam));
      end;// 1014
      1015 : begin //
        _InitActData(message.LParam);
      end;// 1015
// <<<<  


      1099 : begin //    
        _ShowSettings;
      end;// 1099
      1100 : begin //   ""
        _ShowLog(Now,Now);
      end;// 1100
      1101 : begin //     (  TLogData)
        _LogEvent(TLogData(Pointer(message.LParam)^));
      end;// 1101

    end;// case
  end;// if
end;

exports
  OnMainWndProc name 'ONMAINWNDPROC';
begin

end.


